/*
 * ar4c-dcdc - jiffies.c
 *
 * Copyright (C) 2014  Krzysztof Mazur <krzysiek@podlesie.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif
#include <jiffies.h>
#include <compiler.h>
#include <avr/io.h>
#include <avr/sleep.h>
#include <avr/interrupt.h>

unsigned long jiffies = -300 * HZ;

ISR(TIMER0_OVF_vect)
{
	jiffies++;
}

unsigned long get_jiffies(void)
{
	unsigned long ret;

	cli();
	ret = jiffies;
	sei();
	return ret;
}

void sleep_to(unsigned long t)
{
	set_sleep_mode(SLEEP_MODE_IDLE);
	cli();
	while (time_after_eq(t, jiffies)) {
		sleep_enable();
		sei();
		sleep_cpu();
		sleep_disable();
		cli();
	}
	sei();
}

#define TCCR_DIV1		1
#define TCCR_DIV8		2
#define TCCR_DIV64		3
#define TCCR_DIV256		4
#define TCCR_DIV1024		5

void jiffies_init(void)
{
	TCCR0 = TCCR_DIV64;
	TCNT0 = 0;
	TIMSK = BIT(TOIE0);
}
